// Buffer class header
#pragma once

#include "GraphicsResource.h"
#include "VideoMemoryAllocator.h"

namespace nf::client::render {
    enum class BufferType {
        Staging,
        Vertex,
        Index,
        Uniform
    };

    class Buffer : GraphicsResource {
    public:
        Buffer(BufferType type, const VkDevice& device, VideoMemoryAllocator& allocator, const VkCommandPool& commandPool, const VkQueue& queue, void* bufferData, size_t bufferSize);

        const VkBuffer& getHandle() const;
        void* getPointer() const;
        uint32_t getIndicesCount() const;

        ~Buffer();
    private:
        void createBuffer(VkDeviceSize size, VkBufferUsageFlags usage, VkBuffer& buffer);
        void copyBuffer(VkBuffer sourceBuffer, VkBuffer destinationBuffer, VkDeviceSize size, VkCommandPool commandPool, VkQueue queue);

        void destroyBuffer(VkBuffer buffer, VideoMemoryAllocation& allocation);

        VideoMemoryAllocator& m_allocator;

        VkBuffer m_handle;
        VideoMemoryAllocation m_allocation;
        uint32_t m_indexCount;
    };
}
