// VideoMemoryAllocator class header
#pragma once

namespace nf::client::render {
    struct VideoMemoryAllocation {
        VkDeviceMemory memory;
        VkDeviceSize offset;
        void* mappedMemoryPointer;
    };

    struct MemoryBlock {
        VkDeviceMemory memory;
        uint32_t memoryTypeIndex;
        size_t size;
        void* mappedMemoryPointer;

        std::map<VkDeviceSize, VkDeviceSize> allocations;
        //       Offset        Size
    };

    class VideoMemoryAllocator {
    public:
        VideoMemoryAllocator(const VkDevice& device, const VkPhysicalDevice& physicalDevice);

        void allocateForBuffer(VkBuffer buffer, VkMemoryPropertyFlags memoryPropertyFlags, VideoMemoryAllocation& outAllocation);
        void allocateForImage(VkImage image, VkMemoryPropertyFlags memoryPropertyFlags, VideoMemoryAllocation& outAllocation);

        void deallocate(const VideoMemoryAllocation& allocation);
    private:
        void allocate(VkMemoryRequirements memoryRequirements, VkMemoryPropertyFlags memoryPropertyFlags, VideoMemoryAllocation& outAllocation);
        uint32_t findMemoryType(uint32_t typeFilter, VkMemoryPropertyFlags desiredPropertyFlags);

        size_t calculateBlockSize(uint32_t memoryTypeIndex) const;
        void allocateBlock(VkMemoryAllocateInfo& allocateInfo);

        const VkDevice& m_device;
        const VkPhysicalDevice& m_physicalDevice;

        std::vector<MemoryBlock> m_blocks;
    };
}
